/**
 * Some ISA specific functions relevant for measuring 
 * the time some piece of code runs.
 */

/** tic() and toc() follow recommendations from Intel:
 * https://www.intel.com/content/dam/www/public/us/en/documents/white-papers/ia-32-ia-64-benchmark-code-execution-paper.pdf
 */
static inline uint64_t tic() {
    uint64_t hi, lo;
    __asm__ volatile (
        "cpuid\n\t"
        "rdtsc\n\t"
        "mov %%rdx, %0\n\t"
        "mov %%rax, %1\n\t"
        : "=r" (hi), "=r" (lo)
        :: "%rax", "%rbx", "%rcx", "%rdx");
    return (hi << 32) | lo;
}

static inline uint64_t toc() {
    uint64_t hi, lo;
    __asm__ volatile(
        "rdtscp\n\t"
        "mov %%rdx, %0\n\t"  
        "mov %%rax, %1\n\t" 
        "cpuid\n\t"
        : "=r" (hi), "=r" (lo)
        :: "%rax", "%rbx", "%rcx", "%rdx");
    return (hi << 32) | lo;
}

// Flush the memory pointed to by p out of cache
static inline void clflush(void const *p) {
  __asm__ volatile ( "clflush 0(%0)\n" : : "r" (p) : );
}

// Serialize execution
static inline void mfence() {
  __asm__ volatile ( "mfence\n" : : : );
}
